"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.middleware = middleware;
const jose_1 = require("jose");
const errors_js_1 = require("../errors.js");
const utils_js_1 = require("../utils.js");
const authorization_js_1 = require("./authorization.js");
const mock_js_1 = require("./mock.js");
function bind_authorization(authorization_funs, user) {
    const { check_authorization } = authorization_funs;
    async function bound_check_authorization(subject_or_action_or_tuples, action_or_object, object) {
        let tuples;
        if (Array.isArray(subject_or_action_or_tuples)) {
            tuples = subject_or_action_or_tuples.map(tuple => ({
                subject: user,
                ...tuple,
            }));
            return check_authorization(tuples);
        }
        else if (action_or_object !== undefined) {
            if (object === undefined) {
                return check_authorization(user, subject_or_action_or_tuples, action_or_object);
            }
            else {
                return check_authorization(subject_or_action_or_tuples, action_or_object, object);
            }
        }
        else {
            return { result: false, reason: 'Invalid arguments' };
        }
    }
    async function check_authorization_or_fail(subject_or_action_or_tuples, action_or_object, object) {
        let result = await bound_check_authorization(subject_or_action_or_tuples, action_or_object, object);
        if (!Array.isArray(result)) {
            result = [result];
        }
        if (result.some((r) => !r.result)) {
            if (user === 'user:anonymus') {
                throw new errors_js_1.UnauthorizedError();
            }
            else {
                throw new errors_js_1.ForbiddenError();
            }
        }
    }
    async function filter(array, action, key) {
        const authorization_map = await bound_check_authorization(array.map(o => ({ action, object: key(o) })));
        return array.filter((_, idx) => authorization_map[idx].result);
    }
    return {
        ...authorization_funs,
        check_authorization: bound_check_authorization,
        check_authorization_or_fail,
        filter,
    };
}
/**
 * This middleware adds the authorization functions to the request object.
 *
 * The user is read from the `X-Request-Authentication` header. If the header is not set, the user is set to `user:anonymus`.
 * @param config
 * @returns
 */
function middleware(config) {
    if (config === undefined) {
        config = {
            AUTHORIZATION_SERVER: process.env.AUTHORIZATION_SERVER ||
                (0, utils_js_1.die)('Environment variable AUTHORIZATION_SERVER must be set'),
            AUTHORIZATION_PSK: process.env.AUTHORIZATION_PSK ||
                (0, utils_js_1.die)('Environment variable AUTHORIZATION_PSK must be set'),
        };
    }
    let authorization_funs = (0, authorization_js_1.authorization_functions)(config);
    return ((req, _res, next) => {
        const user = req.header('X-Request-Authentication') ?? 'user:anonymus';
        let user_id = user;
        try {
            user_id = (0, jose_1.decodeJwt)(user).sub ?? 'user:anonymus';
        }
        catch (e) {
            //ignore
        }
        if (req.app.authorization_mock) {
            authorization_funs = (0, mock_js_1.mock_authorization_functions)(req.app.authorization_mock);
        }
        req.authorization = {
            ...bind_authorization(authorization_funs, user),
            user: user_id,
        };
        next();
    });
}
//# sourceMappingURL=middleware.js.map